"""
    an image processing technique to produce a 1D signal for each pixel : this 1D signal is obtained by projecting the neighborhood of this pixel on a set of bases (each base is used as a convilution kernel)
    https://subversion.ipr.univ-rennes1.fr/repos/main/projects/antipode/src/python/antipode/texori.py
"""
from imageengine import IImageEngine, PixelType

class IProjectorBase(object):
    """
        a set of projectors

        a projector is a grey level image, in which each pixel value is acting as a weight
    """
    def __init__(self):
        pass

    def get_num_projectors(self):
        pass
        
    def get_projector_kernel(self, projector_index):

        pass

def create_circle_image(image_size, circle_radius, circle_pos, circle_thickness, background_value=0.0, circle_value=1.0):
    """
    :param dict image_size:
    """
    ie = IImageEngine.get_instance()
    width = image_size['width']
    height = image_size['height']
    assert isinstance(width, int)
    assert isinstance(height, int)
    circle_pos_x = float(circle_pos['x'])
    circle_pos_y = float(circle_pos['y'])
    r_min = max(0.0, circle_radius - circle_thickness * 0.5)
    r_max = circle_radius + circle_thickness * 0.5
    r_min_square = r_min * r_min
    r_max_square = r_max * r_max
    image = ie.create_image(width=width, height=height, pixel_type=PixelType.F32)
    for y in range(height):
        for x in range(width):
            dx = x - circle_pos_x
            dy = y - circle_pos_y
            r_square = (dx * dx + dy * dy)
            if r_min_square - 0.001 <= r_square < r_max_square:
                pixel_value = circle_value
            else:
                pixel_value = background_value
            image.set_pixel(x, y, pixel_value)
    return image

class CircularSymmetryProjectorBase(IProjectorBase):
    """
        generates a base of circles 
    """

    def __init__(self, max_radius, oversampling_scale=2):
        """
        :param int max_radius: the biggest circle radius in the set of projectors
        :param int oversampling_scale: oversampling is used to generate antialased circles. The higher this value, the better the quality of antialiasing is
        """
        super(CircularSymmetryProjectorBase, self).__init__()
        assert max_radius > 0
        assert oversampling_scale > 0
        self.max_radius = max_radius
        self.oversampling_scale = oversampling_scale
        self.num_projectors = int(max_radius) + 1

    def get_num_projectors(self):
        return self.num_projectors
        
    def get_projector_kernel(self, projector_index):
        assert projector_index < self.get_num_projectors()
        oversampling_is_handled = False  # TODO: handle oversampling to increase quality
        if not oversampling_is_handled and self.oversampling_scale != 1  :
            raise NotImplementedError("at the moment, oversampling is not yet implemented")  
        radius = projector_index
        image_size = int(self.max_radius) * 2 + 1
        circle_pos = {'x': self.max_radius, 'y': self.max_radius}
        oversampled_circle = create_circle_image(
            image_size={'width': image_size * self.oversampling_scale, 'height': image_size*self.oversampling_scale},
            circle_radius=radius * self.oversampling_scale,
            circle_pos={'x': self.max_radius* self.oversampling_scale, 'y': self.max_radius* self.oversampling_scale},
            circle_thickness=1 * self.oversampling_scale)
        if self.oversampling_scale == 1:
            circle_image = oversampled_circle
        else:
            if oversampling_is_handled:
                circle_image = oversampled_circle.resample(width=image_size, height=image_size)
        mean_value = circle_image.get_mean_value()
        assert mean_value > 0.0, "unexpected mean value of this circle image : this circle might be empty, or worse, have negative values, which doesn't make sense"
        print(type(mean_value))
        num_pixels = image_size * image_size
        sum_of_pixel_values = mean_value * num_pixels
        # we want each circle to have a total weight of 1.0, regardless their radius
        circle_image.scale_values(1.0/sum_of_pixel_values)
        

        
        anchor_point = {'x': int(circle_pos['x']), 'y': int(circle_pos['y'])}
        return circle_image, anchor_point


class CircularSymmetryDetector:
    def __init__(self, max_radius):
        """
        :para float max_radius:
        """
        self.max_radius = max_radius

    def compute_radial_profiles(self, src_image):
        """ Computes for each pixel the radial profile (with this pixel as center)

        :param IImage src_image:
        :rtype IHyperstack: each radial profile is stored in the channel coordinate of the hyperstack
        """
        ie = IImageEngine.get_instance()
        ie.debugger.on_image(src_image, 'src_image')
        projector_base = CircularSymmetryProjectorBase(self.max_radius, oversampling_scale=1)
        radial_profile_image = ie.create_hyperstack(width=src_image.get_width(), height=src_image.get_height(), num_channels=projector_base.get_num_projectors(), num_slices=1, num_frames=1, pixel_type=PixelType.F32)
        for projector_index in range(projector_base.get_num_projectors()):
            projector, center_of_filter = projector_base.get_projector_kernel(projector_index)
            ie.debugger.on_image(projector, 'projector_%d' % (projector_index))
            print(type(center_of_filter))
            projection = ie.filter2D(src_image, dst_type=PixelType.F32, kernel=projector, anchor=(center_of_filter['x'], center_of_filter['y']))
            ie.debugger.on_image(projection, 'projection_%d' % (projector_index))
            radial_profile_image.set_image(projection, frame_index=0, slice_index=0, channel_index=projector_index)
        return radial_profile_image
